@@ -24,6 +24,7 @@ import six
 from pyemma.coordinates.data._base.iterable import Iterable
 from pyemma.coordinates.data._base.random_accessible import TrajectoryRandomAccessible
 from pyemma.util import config
+from pyemma.util.annotators import deprecated
 from six import string_types
 import os
 
@@ -135,7 +136,7 @@ class DataSource(Iterable, TrajectoryRandomAccessible):
 
                 raise ValueError("Input data has different dimensions ({dims})!"
                                  " Files grouped by dimensions: {groups}".format(dims=res.keys(),
-                                                                                  groups=res))
+                                                                                 groups=res))
 
             self._ndim = ndims[0]
             self._lengths = lengths
@@ -165,14 +166,22 @@ class DataSource(Iterable, TrajectoryRandomAccessible):
         """
         return self
 
-    def number_of_trajectories(self):
+    def number_of_trajectories(self, stride=None):
         r""" Returns the number of trajectories.
 
+        Parameters
+        ----------
+        stride: None (default) or np.ndarray
+
         Returns
         -------
             int : number of trajectories
         """
-        return self._ntraj
+        if not IteratorState.is_uniform_stride(stride):
+            n = len(np.unique(stride[:, 0]))
+        else:
+            n = self._ntraj
+        return n
 
     def trajectory_length(self, itraj, stride=1, skip=None):
         r"""Returns the length of trajectory of the requested index.
@@ -184,6 +193,8 @@ class DataSource(Iterable, TrajectoryRandomAccessible):
         stride : int
             return value is the number of frames in the trajectory when
             running through it with a step size of `stride`.
+        skip: int or None
+            skip n frames.
 
         Returns
         -------
@@ -192,12 +203,28 @@ class DataSource(Iterable, TrajectoryRandomAccessible):
         if itraj >= self._ntraj:
             raise IndexError("given index (%s) exceeds number of data sets (%s)."
                              " Zero based indexing!" % (itraj, self._ntraj))
-        if isinstance(stride, np.ndarray):
+        if not IteratorState.is_uniform_stride(stride):
             selection = stride[stride[:, 0] == itraj][:, 0]
             return 0 if itraj not in selection else len(selection)
         else:
             return (self._lengths[itraj] - (0 if skip is None else skip) - 1) // int(stride) + 1
 
+    def n_chunks(self, chunksize, stride=1, skip=0):
+        """ how many chunks an iterator of this sourcde will output, starting (eg. after calling reset())
+
+        Parameters
+        ----------
+        chunksize
+        stride
+        skip
+        """
+        if chunksize != 0:
+            chunks = int(sum((ceil(l / float(chunksize))
+                          for l in self.trajectory_lengths(stride=stride, skip=skip))))
+        else:
+            chunks = self.number_of_trajectories(stride)
+        return chunks
+
     def trajectory_lengths(self, stride=1, skip=0):
         r""" Returns the length of each trajectory.
 
@@ -213,8 +240,9 @@ class DataSource(Iterable, TrajectoryRandomAccessible):
         -------
         array(dtype=int) : containing length of each trajectory
         """
-        n = self.number_of_trajectories()
-        if isinstance(stride, np.ndarray):
+        n = self.ntraj
+
+        if not IteratorState.is_uniform_stride(stride):
             return np.fromiter((self.trajectory_length(itraj, stride)
                                 for itraj in range(n)),
                                dtype=int, count=n)
@@ -237,8 +265,8 @@ class DataSource(Iterable, TrajectoryRandomAccessible):
         n_frames_total : int
             total number of frames.
         """
-        if isinstance(stride, np.ndarray):
-            return stride.shape[0]
+        if not IteratorState.is_uniform_stride(stride):
+            return len(stride)
 
         return sum(self.trajectory_lengths(stride=stride, skip=skip))
 
@@ -250,7 +278,7 @@ class IteratorState(object):
 
     def __init__(self, skip=0, chunk=0, return_trajindex=False, ntraj=0, cols=None):
         self.skip = skip
-        self.chunk = chunk
+        self._chunk = chunk
         self.return_trajindex = return_trajindex
         self.itraj = 0
         self.ntraj = ntraj
@@ -263,6 +291,15 @@ class IteratorState(object):
         self.trajectory_lengths = None
         self.ra_indices_for_traj_dict = {}
         self.cols = cols
+        self.current_itraj = 0
+
+    @property
+    def chunk(self):
+        return self._chunk
+
+    @chunk.setter
+    def chunk(self, value):
+        self._chunk = value
 
     def ra_indices_for_traj(self, traj):
         """
@@ -349,17 +386,14 @@ class DataSourceIterator(six.with_metaclass(ABCMeta)):
         return self.state.is_stride_sorted()
 
     @property
-    def _n_chunks(self):
+    def n_chunks(self):
         """ rough estimate of how many chunks will be processed """
-        if self.chunksize != 0:
-            if not DataSourceIterator.is_uniform_stride(self.stride):
-                chunks = ceil(len(self.stride[:, 0]) / float(self.chunksize))
-            else:
-                chunks = sum((ceil(l / float(self.chunksize))
-                              for l in self.trajectory_lengths()))
-        else:
-            chunks = self.number_of_trajectories()
-        return int(chunks)
+        return self._data_source.n_chunks(self.chunksize, stride=self.stride, skip=self.skip)
+
+    @property
+    @deprecated("use n_chunks")
+    def _n_chunks(self):
+        return self.n_chunks
 
     def number_of_trajectories(self):
         return self._data_source.number_of_trajectories()
@@ -378,12 +412,22 @@ class DataSourceIterator(six.with_metaclass(ABCMeta)):
         """ closes the reader"""
         pass
 
+    @abstractmethod
+    def _select_file(self, itraj):
+        """ opens the next file defined by itraj.
+
+        Parameters
+        ----------
+        itraj : int
+            index of trajectory to open.
+        """
+        pass
+
     def reset(self):
         """
         Method allowing to reset the iterator so that it can iteration from beginning on again.
         """
-        self._t = 0
-        self._itraj = 0
+        self._select_file(0)
 
     @property
     def pos(self):
@@ -618,6 +662,7 @@ class DataSourceIterator(six.with_metaclass(ABCMeta)):
             while (self._itraj not in self.traj_keys or self._t >= self.ra_trajectory_length(self._itraj)) \
                     and self._itraj < self.number_of_trajectories():
                 self._itraj += 1
+            self._select_file(self._itraj)
         # we have to obtain the current index before invoking next_chunk (which increments itraj)
         self.state.current_itraj = self._itraj
         self.state.pos = self.state.pos_adv
@@ -647,6 +692,14 @@ class DataSourceIterator(six.with_metaclass(ABCMeta)):
                 (not self.return_traj_index and len(X) == 0) or (self.return_traj_index and len(X[1]) == 0)
         ):
             X = self._it_next()
+        if config.coordinates_check_output:
+            array = X if not self.return_traj_index else X[1]
+            if not np.all(np.isfinite(array)):
+                # determine position
+                start = self.pos
+                msg = "Found invalid values in chunk in trajectory index {itraj} at chunk [{start}, {stop}]" \
+                    .format(itraj=self.current_trajindex, start=start, stop=start+len(array))
+                raise InvalidDataInStreamException(msg)
         return X
 
     def __iter__(self):
@@ -659,3 +712,14 @@ class DataSourceIterator(six.with_metaclass(ABCMeta)):
         self.close()
         return False
 
+    def __repr__(self):
+        return "[{name} chunk={chunk}, stride={stride}, skip={skip}]".format(
+            name=self.__class__.__name__,
+            chunk=self.chunksize,
+            stride=self.stride,
+            skip=self.skip
+        )
+
+
+class InvalidDataInStreamException(Exception):
+    """Data stream contained NaN or (+/-) infinity"""
